@@ -53,12 +53,11 @@ pub fn find(text: &str, target: &str) -> usize {
   text.find(target).unwrap_or_else(|| panic!("`{}` not in `{}`.", target, text))
 }
 
-pub fn read<'a, A>(parser: Parser<'a, A>, code: &'a str) -> A {
+pub fn read<'a, A>(parser: Parser<'a, A>, code: &'a str) -> Result<A, String> {
   match parser(State { code, index: 0 }) {
-    Ok((_, value)) => value,
+    Ok((_, value)) => Ok(value),
     Err(msg) => {
-      println!("{}", msg);
-      panic!("No parse.");
+      Err(msg)
     }
   }
 }
@@ -233,9 +232,19 @@ pub fn guard<'a, A: 'a>(
   }
 }
 
-/// Applies optional parsers in sequence.
-/// Returns the first that succeeds.
-/// Aborts if none succeed.
+pub fn parser_or<'a>(parsers: &[Parser<'a, bool>], state: State<'a>) -> Answer<'a, bool> {
+  for parser in parsers {
+    let (state, matched) = parser(state)?;
+    if matched {
+      return Ok((state, true));
+    }
+  }
+  Ok((state, false))
+}
+
+// Applies optional parsers in sequence.
+// Returns the first that succeeds.
+// If none succeeds, aborts.
 pub fn grammar<'a, A: 'a>(
   name: &'static str,
   choices: &[Parser<'a, Option<A>>],
@@ -376,9 +385,7 @@ pub fn highlight(from_index: usize, to_index: usize, code: &str) -> String {
   let close_color = "\x1b[0m";
   let mut from_line = 0;
   let mut to_line = 0;
-  for (i, c) in
-    code.chars().enumerate().filter(|(_, c)| c == &'\n').take_while(|(i, _)| i < &to_index)
-  {
+  for (i, c) in code.char_indices().filter(|(_, c)| c == &'\n').take_while(|(i, _)| i < &to_index) {
     if i < from_index {
       from_line += c.len_utf8();
     }
