@@ -246,6 +246,6 @@ pub fn as_code(mem: &Worker, comp: &Option<rb::RuleBook>, host: u64) -> String {
   go(ctx, &mut stacks, term, 0)
 }
 
-pub fn as_term(mem: &Worker, comp: &Option<rb::RuleBook>, host: u64) -> Box<lang::Term> {
+pub fn as_term(mem: &Worker, comp: &Option<rb::RuleBook>, host: u64) -> Result<Box<lang::Term>, String> {
   lang::read_term(&as_code(mem, comp, host))
 }
